"""
    This Sub Package contain all wdigets elements for the
    Text User Interface PyeMenu has, these elements are:
    1. Menu -> For just one selection Menu
    2. Checkboxlist -> For a checkbox list with boolean choice
    3. Form -> A Simple widget to create Forms

    This package was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*
    
    PyeMenu version 1.0.1
"""
from .menus import Menu
from .checkboxlists import Checkboxlist
from .forms import Form
