"""
    This Sub Package contain all components for being used insides widgets
    for develop a Text User Interface with PyeMenu, these components are:
        1. Text
        2. Title
        3. Checkbox
        4. Entry
        5. Button
    
    PyeMenu version 1.0.1
    This package was developed by Freire Alexander Palomino Palma
    *Copyright (c) 2014-2023 Freire Alexander Palomino Palma*
"""
from .texts import Text
from .titles import Title
from .checkboxs import Checkbox
from .entries import Entry
from .buttons import Button